/*
  Bluetooth CW Keyer - Master Unit

  by Glen Popiel - KW5GP

  Allows you to use standard Bluetooth HC-05 modules to remote
  to remotely connect a set of CW keyer paddles

  Sketch is written for ZS-040 Bluetooth Module
  Firmware Version: 3.0-2017-0601
*/

#define debug // uncomment to enable debug output

#include <AltSoftSerial.h>  // Include the software serial library

// AltSoftSerial always uses these pins:
//
// Board          Transmit  Receive   PWM Unusable
// -----          --------  -------   ------------
// Teensy 3.0 & 3.1  21        20         22
// Teensy 2.0         9        10       (none)
// Teensy++ 2.0      25         4       26, 27
// Arduino Uno        9         8         10
// Arduino Leonardo   5        13       (none)
// Arduino Mega      46        48       44, 45
// Wiring-S           5         6          4
// Sanguino          13        14         12

#define serial_baud 38400 // Set the Serial Monitor baud rate
#define bt_baud 38400 // Set the Bluetooth module baud rate
#define BT_PWR 3  // Define the Bluetooth module Power Control Pin
#define BT_EN 4   // Define the Bluetooth module Enable Pin
#define Dit_pin 6 // Define the Dit Relay drive pin
#define Dah_pin 7 // Define the Dah Relay drive pin
#define command_delay 1000  // Delay between the Bluetooth commands

bool run_once = true;
String rx_line;
bool receiving;
char c;

AltSoftSerial altSerial;  // Instantiate the AltSoftSerial instance

void setup()
{
  pinMode(Dit_pin, OUTPUT); // Setup the Dit Relay pin for output
  pinMode(Dah_pin, OUTPUT); // Setup the Dah Relay pin for output
  pinMode(BT_PWR, OUTPUT);  // Setup the Bluetooth module Power Control pin for output
  pinMode(BT_EN, OUTPUT);   // Setup the Bluetooth Enable pin for output

  digitalWrite(Dit_pin, LOW); // Turn off both the Dit and Dah relays
  digitalWrite(Dah_pin, LOW);

#ifdef debug
  Serial.begin(serial_baud);
  while (!Serial) ; // wait for Arduino Serial Monitor to open
  Serial.println("HC-05 Master Setup Begin");
  delay(command_delay);
#endif

  altSerial.begin(bt_baud); // Start the Bluetooth module serial connection
  delay(command_delay);

  // Turn on Bluetooth module PWR and EN pins

#ifdef debug
  Serial.println("Set HC-05 Enable");
  delay(command_delay);
#endif

  digitalWrite(BT_EN, HIGH);  // Turn on the Enable pin
  delay(command_delay);

#ifdef debug
  Serial.println("Turn on HC-05 Power");
  delay(command_delay);
#endif

  digitalWrite(BT_PWR, HIGH); // Turn on the power to the Bluetooth module
  delay(command_delay);

#ifdef debug
  Serial.println("HC-05 Ready for commands");
  delay(command_delay);
#endif

  read_bt();  // Read any responses from the Bluetooth module
  delay(command_delay);
}

void loop()
{
  if (run_once) // On the first pass, configure the Bluetooth module
  {
    digitalWrite(Dit_pin, LOW); // Turn off the dit and dah relays
    digitalWrite(Dah_pin, LOW);

#ifdef debug
    Serial.println("Configure HC-05 Bluetooth module");
#endif

#ifdef debug
    Serial.println("Set Bluetooth Module name to HC-05");
#endif

    delay(command_delay);
    altSerial.println("AT+NAME=HC-05"); // Send the Set Name command
    delay(command_delay);  // Read any responses from the Bluetooth module
    read_bt();  // Read any responses from the Bluetooth module

#ifdef debug
    Serial.println("Set HC-05 Role to 1");
#endif

    delay(command_delay);
    altSerial.println("AT+ROLE=1"); // Send the Master Role command
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module

#ifdef debug
    Serial.println("Set HC-05 Connection Mode to 1");
#endif

    altSerial.println("AT+CMODE=1");  // Send the Connect to Any command
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module

#ifdef debug
    Serial.println("Set HC-05 Baud Rate to 38400");
#endif

    altSerial.println("AT+UART=38400,0,0"); // Send the Baud Rate command
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module

#ifdef debug
    // Display the settings
    Serial.println("Bluetooth module Configuration Settings");
    altSerial.println("AT+NAME"); //Display the module name
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    altSerial.println("AT+VERSION");  // Display the Firmware Version
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    altSerial.println("AT+ADDR"); // Display the Bluetooth module Address
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    altSerial.println("AT+ROLE"); // Display the Bluetooth Role
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    altSerial.println("AT+CMODE");  // Display the Bluetooth Connection Mode
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    altSerial.println("AT+UART"); // Display the Bluetooth module Baud Rate
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    Serial.println("Configuration Complete - Starting Link");
#endif

    digitalWrite(BT_EN, LOW);    // Turn off the Enable pin
    delay(1000);

    // Start the Bluetooth module communication link
    altSerial.println("AT+RESET");  // Send the Bluetooth module Reset command
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    flush_bt(); // Flush any additional repsonses

    run_once = false;
  }

  read_bt();  // Read any responses from the Bluetooth module
}

void read_bt()  // Function to read the Bluetooth module responses
{
  rx_line = "";
  do
  {
    if (altSerial.available())
    {
      receiving = true;
      c = altSerial.read(); // Get the incoming character

      if (!run_once)  // Process the incoming character only if we're not doing configuration
      {
        switch (c)
        {
          case '0': // No CW paddle pressed - turn off both relays
            digitalWrite(Dit_pin, LOW);
            digitalWrite(Dah_pin, LOW);

#ifdef debug
            Serial.println("Paddles Released");
#endif

            break;

          case '1': // Dit paddle press received - turn on dit relay
            digitalWrite(Dit_pin, HIGH);

#ifdef debug
            Serial.println("Rx Dit");
#endif

            break;

          case '2': // Dah paddle press received - turn on dah relay
            digitalWrite(Dah_pin, HIGH);

#ifdef debug
            Serial.println("Rx Dah");
#endif

            break;
        }
      } else
      {

#ifdef debug
        Serial.print(c);  // Otherwise print the character we received
#endif

      }
    }
  } while (altSerial.available());
}

void flush_bt() // Function to flush the received character buffer
{
  do
  {
    c = altSerial.read(); // Just keep reading until there are no more characters in the buffer
  } while (altSerial.available());
  c = ' ';
}
