#include <AltSoftSerial.h>

// AltSoftSerial always uses these pins:
//
// Board          Transmit  Receive   PWM Unusable
// -----          --------  -------   ------------
// Teensy 3.0 & 3.1  21        20         22
// Teensy 2.0         9        10       (none)
// Teensy++ 2.0      25         4       26, 27
// Arduino Uno        9         8         10
// Arduino Leonardo   5        13       (none)
// Arduino Mega      46        48       44, 45
// Wiring-S           5         6          4
// Sanguino          13        14         12

AltSoftSerial altSerial;

// #define debug // Uncomment to enable debug output on Serial Monitor

#define serial_baud 38400
#define bt_baud 38400
#define BT_PWR 3  // Define the Bluetooth module Power Control Pin
#define BT_EN 4   // Define the Bluetooth module Enable Pin
#define dit_pin 6 // Define the Dit Relay drive pin
#define dah_pin 7 // Define the Dah Relay drive pin
#define command_delay 1000  // Delay between the Bluetooth commands
#define debounce_delay 20 // CW Paddle debounce delay

bool run_once = true;
String rx_line;
bool receiving, dit_switch, dah_switch;
bool dit_flag = false;
bool dah_flag = false;
char c;

void setup()
{
  pinMode(dit_pin, INPUT_PULLUP); // Setup the Dit paddle pin for inout with pullup resistor
  pinMode(dah_pin, INPUT_PULLUP); // Setup the Dah paddle pin for inout with pullup resistor
  pinMode(BT_PWR, OUTPUT);  // Setup the Bluetooth module Power Control pin for output
  pinMode(BT_EN, OUTPUT);   // Setup the Bluetooth Enable pin for output

#ifdef debug
  Serial.begin(serial_baud);
  while (!Serial) ; // wait for Arduino Serial Monitor to open
  Serial.println("HC-05 Setup Begin");
#endif

  altSerial.begin(bt_baud); // Start the Bluetooth module serial connection
  delay(1000);

  // Turn on Bluetooth PWR and EN pins

#ifdef debug
  Serial.println("Set HC-05 Enable");
#endif

  digitalWrite(BT_EN, HIGH);
  delay(command_delay);

#ifdef debug
  Serial.println("Turn on HC-05 Power");
#endif

  digitalWrite(BT_PWR, HIGH);

#ifdef debug
  Serial.println("HC-05 Ready for commands");
#endif
}

void loop()
{
  if (run_once)
  {
    delay(command_delay);
#ifdef debug
    Serial.println("Set Bluetooth Module name to HC-05");
#endif

    delay(command_delay);
    altSerial.println("AT+NAME=HC-05"); // Send the Set Name command
    delay(command_delay);  // Read any responses from the Bluetooth module
    read_bt();  // Read any responses from the Bluetooth module

#ifdef debug
    Serial.println("Set HC-05 Role to 0");
#endif

    altSerial.println("AT+ROLE=0"); // Send the Slave Role command
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module

#ifdef debug
    Serial.println("Set HC-05 Connection Mode to 1");
#endif

    altSerial.println("AT+CMODE=1");  // Send the Connect to Any command
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module

#ifdef debug
    Serial.println("Set HC-05 Baud Rate to 38400");
#endif

    altSerial.println("AT+UART=38400,0,0"); // Send the Baud Rate command
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module

#ifdef debug
    // Display the settings
    Serial.println("Bluetooth module Configuration Settings");
    altSerial.println("AT+NAME"); //Display the module name
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    altSerial.println("AT+VERSION");  // Display the Firmware Version
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    altSerial.println("AT+ADDR"); // Display the Bluetooth module Address
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    altSerial.println("AT+ROLE"); // Display the Bluetooth Role
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    altSerial.println("AT+CMODE");  // Display the Bluetooth Connection Mode
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    altSerial.println("AT+UART"); // Display the Bluetooth module Baud Rate
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module
    Serial.println("Configuration Complete - Starting Link");
#endif

    delay(command_delay);
    digitalWrite(BT_EN, LOW);    // Turn off Enable pin
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module

    // Start communication with Master
    altSerial.println("AT+RESET");    // Send the Bluetooth module Reset command
    delay(command_delay);
    read_bt();  // Read any responses from the Bluetooth module

    run_once = false;

    //  Send an ASCII "0" (zero) to turn off relays on Master
    altSerial.println("0");
  }

  // Read and send the switch closure information

  dit_switch = !(digitalRead(dit_pin)); // Read the Dit Paddle Input
  dah_switch = !(digitalRead(dah_pin)); // Read the Dah Paddle Input

  if (dit_switch && !dit_flag)  // Handle a Dit paddle press
  {
    altSerial.print("1"); // Send an ASCII "1" to the Master
    dit_flag = true;  // Only send one character until it's released
    delay(debounce_delay);

#ifdef debug
    Serial.println("Tx Dit");
#endif

  }

  if (dah_switch && !dah_flag)  // Handle a Dah paddle press
  {
    altSerial.print("2"); // Send an ASCII "2" to the Master
    dah_flag = true;  // Only send one character until it's released
    delay(debounce_delay);

#ifdef debug
    Serial.println("Tx Dah");
#endif

  }

// Send an ASCII "0" after either paddle was pressed and then released
  if ((!dit_switch && dit_flag) || (!dah_switch && dah_flag))
  {
    altSerial.print("0");
    dit_flag = false;
    dah_flag = false;
    delay(debounce_delay);

#ifdef debug
    Serial.println("Unkeying");
#endif
  }
}

void read_bt()  // Function to read the Bluetooth module responses
{
  rx_line = "";
  do
  {
    if (altSerial.available())
    {
      receiving = true;
      c = altSerial.read(); // Get the incoming character

#ifdef debug
      Serial.print(c);
#endif

      rx_line = rx_line + c;
    }
  } while (altSerial.available());
}
