/*
  CW Beacon/Foxhunt Keyer
  Author Glen Popiel, KW5GP
  Uses Morse Library by Author Erik Linder - errors corrected by Glen Popiel KW5GP
  Beacon mode repeats message at regular intervals
  Foxhunt Mode repeats message for repeat time at interval time (seconds)
*/

#include <Morse.h>
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library
//#include <SPI.h>  // SPI library so we can communicate with the TFT display

#define beacon_msg "DE KW5GP Beacon"
#define key_pin 3  // Pin for CW tone
#define mode_pin 6 // Pin for the Mode switch
#define beep_pin 5  // Pin for PTT/Rig Key
#define key_speed 13  // CW send speed
#define beacon_interval 10// Transmit every 10 seconds in foxuhunt mode
#define foxhunt_interval 60  // Transmit every 60 seconds in foxuhunt mode
int foxhunt_repeat = 30; // Repeat message time in seconds - must be a minimum of 1
int mode = 0; // Mode 0 = Beacon 1 = Foxhunt
int mode_switch;
int last_mode_switch = 99;
int interval = 0;     // Timer interval in seconds
boolean ptt = LOW;

#define TFT_CS     10  // Assign the TFT CS to pin 10
#define TFT_RST    8  // Assign the TFT RST to pin 7
#define TFT_DC     9  // Assign the TFT DC to pin 6

#define tft_delay 10  // set the TFT command delay to 10ms


const char beacon_call[] = beacon_msg;  // Transfer the message definition to a character array
long end_time;  // Foxhunt timing
char c;  // Character to send
bool first_pass = true;  // Flag to tell us this is the first time through
unsigned long offset = 0L;  // Timer value

Morse morse(beep_pin, key_speed, 1); // Set up the Morse Library for use
int msg_length;  // variable to hold the size of the message text array

// Instantiate the TFT Display
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC, TFT_RST);  // Initialize the TFT display instance

// Timer function - returns the time in seconds since last Timer Reset
unsigned long Timer()
{
  return (millis() - offset) / 1000; // return time in seconds
}

// Timer Reset function
void TimerReset(unsigned long val = 0L)
{
  offset = millis() - val;
}

void setup()
{
  tft.initR(INITR_18BLACKTAB);   // initialize a 1.8" TFT with ST7735S chip, black tab
  delay(tft_delay);
  clear_display(); // Clear the display - fill with BLACK background
  delay(tft_delay);
  tft.setRotation(1); // Set the screen rotation
  delay(tft_delay);
  tft.setTextWrap(false); // Turn off Text Wrap
  delay(tft_delay);
  tft.setTextSize(3); // Set the Font Size
  delay(tft_delay);
  tft.setTextColor(ST7735_WHITE); //Set the Text Color
  delay(tft_delay);
  tft.setCursor(40, 10);  //Set the Cursor and display the startup screen
  delay(tft_delay);
  tft.print("KW5GP");
  delay(tft_delay);
  tft.setTextSize(2);
  delay(tft_delay);
  tft.setCursor(40, 60);
  delay(tft_delay);
  tft.print("Foxhunt");
  delay(tft_delay);
  tft.setCursor(65, 80);
  delay(tft_delay);
  tft.print("and");
  delay(tft_delay);
  tft.setCursor(10, 100);
  delay(tft_delay);
  tft.print("Beacon Keyer");
  delay(tft_delay);

  delay(5000);
  msg_length = (sizeof(beacon_call)) - 1; // Calculate the size of the message text array
  pinMode(key_pin, OUTPUT);           // set PTT/Key pin to output
  pinMode(mode_pin, INPUT_PULLUP);  // Set the Mode pin for input with pullup
  digitalWrite(key_pin, LOW); // Turn off the PTT/Key relay

  TimerReset();  // Reset the Timer to zero
  check_mode();
  delay(5000);  // Wait 5 seconds
}  // End Setup Loop

void loop()  // Main Program Loop
{


  if (mode == 0)  // Set the Key mode and LED for Beacon or Foxhunt
  {
    // Set Beacon Mode
    Morse(key_pin, key_speed, 0);    // Set up to key the Relay
  } else
  {
    // Set Foxhunt Mode
    Morse(beep_pin, key_speed, 1);  // Set up to send modulated CW
  }

  if (mode == 0)  // Set the timer interval
  {
    interval = beacon_interval;
  }
  else
  {
    interval = foxhunt_interval;
  }

  if (Timer() > interval || first_pass)  // Send if the Timer has expired or if this is the first time through
  {
    first_pass = false;  // Set the flag to off

    end_time = Timer() + foxhunt_repeat;  // If we're in Foxhunt mode, repeat the message until the repeat time has expired

    ptt = HIGH;
    digitalWrite(key_pin, HIGH);  // Key the PTT
    update_display();
    while (Timer() < end_time) // Check to make sure repeat timer has not expired (Foxhunt Mode)
    {

      for (int x = 0; x < msg_length; x++)  // Send the message in the beacon_call character array one character at a time
      {
        c = beacon_call[x];  // Get the next letter to send
        morse.send(c);  // Send it in CW
      }

      if (mode == 1)  // Send a space if we're in Foxhunt mode to separate messages
      {
        morse.send(char(32));
        delay(1000);

      }
      else
      {
        end_time = Timer() - 1;
      }


    }
    ptt = LOW;
    update_display();
    digitalWrite(key_pin, LOW);  // Make sure the PTT is off
    TimerReset();  // Reset the timer to restart the Interval between messages
  }
  check_mode();
}  // End of Loop

void check_mode() // Reads the mode switch and sets the mode
{
  mode_switch = !(digitalRead(mode_pin));
  if (mode_switch != last_mode_switch)
  {
    mode = !(mode_switch);
    update_display();
    last_mode_switch = mode_switch;
  }
}

void update_display()  // Updates the mode on the display
{
  clear_display();

  delay(tft_delay);
  if (mode == 0)
  {
    tft.setTextSize(2); // Set the Font Size
    delay(tft_delay);
    tft.setCursor(45, 5);
    delay(tft_delay);
    tft.print("Beacon");
    delay(tft_delay);
    tft.setCursor(10, 80);
    delay(tft_delay);
    tft.setTextSize(1); // Set the Font Size
    delay(tft_delay);
    tft.print("TX Interval: ");
    delay(tft_delay);
    tft.print(beacon_interval);
    delay(tft_delay);
    tft.print(" Seconds");
    delay(tft_delay);
    tft.setTextSize(2); // Set the Font Size
    delay(tft_delay);
  }
  else
  {
    tft.setTextSize(2); // Set the Font Size
    delay(tft_delay);
    tft.setCursor(40, 5);
    delay(tft_delay);
    tft.print("Foxhunt");
    delay(tft_delay);
    tft.setTextSize(1); // Set the Font Size
    delay(tft_delay);
    tft.setCursor(20, 80);
    delay(tft_delay);
    tft.print("TX On Time: ");
    delay(tft_delay);
    tft.print(foxhunt_repeat);
    delay(tft_delay);
    tft.print(" Seconds");
    delay(tft_delay);
    tft.setCursor(15, 90);
    delay(tft_delay);
    tft.print("TX Off Time: ");
    delay(tft_delay);
    tft.print(foxhunt_interval);
    delay(tft_delay);
    tft.print(" Seconds");
    delay(tft_delay);
    tft.setTextSize(2); // Set the Font Size
    delay(tft_delay);
  }
  tft.setTextSize(2); // Set the Font Size
  delay(tft_delay);
  tft.setCursor(60, 30);
  delay(tft_delay);
  tft.print("Mode");
  delay(tft_delay);
  tft.setTextSize(1); // Set the Font Size
  delay(tft_delay);
  tft.setCursor(35, 70);
  delay(tft_delay);
  tft.print("CW Speed: ");
  delay(tft_delay);
  tft.print(key_speed);
  delay(tft_delay);
  tft.print(" wpm");
  delay(tft_delay);
}

void clear_display()
{
  if (ptt)
  {
    tft.fillScreen(ST7735_RED);
    delay(tft_delay);
    tft.setTextSize(1); // Set the Font Size
    delay(tft_delay);
    tft.setCursor(45, 115);
    delay(tft_delay);
    tft.print("Transmitting");
    delay(tft_delay);
    tft.setTextSize(2); // Set the Font Size
    delay(tft_delay);
  }
  else
  {
    tft.fillScreen(ST7735_BLUE);
    delay(tft_delay);
    tft.setTextSize(2); // Set the Font Size
    delay(tft_delay);
  }
}
