/*
  Arduino TouchTone Decoder
  Author Glen Popiel, KW5GP
  ------------------------------------------
  Uses the MT8870 TouchTone Decoder Chip
*/

//#define debug // define debug to enable debug Serial Monitor output
#define baud_rate 115200

#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library

#define TFT_CS     10  // Assign the TFT CS to pin 10
#define TFT_RST    8  // Assign the TFT RST to pin 7
#define TFT_DC     9  // Assign the TFT DC to pin 6

#define StD_pin 7
#define Q1_pin 3
#define Q2_pin 4
#define Q3_pin 5
#define Q4_pin 6
#define relay_pin 2

#define text_font_size 3  // The TFT text display font size
#define relay_font_size 2
#define status_font_size 1  // The font size for the status line
#define tft_line_length 8

bool previous_state = LOW;
bool Tone_Decoded = LOW;
bool relay_active = LOW;

int Q1;
int Q2;
int Q3;
int Q4;
int digit_value;
char digit;
char turn_on_relay = '*';
char turn_off_relay = '#';

String tft_data = "";  // String variable to hold tft text data

#define tft_delay 10  // set the TFT command delay to 10ms

// Instantiate the TFT Display
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC, TFT_RST);  // Initialize the TFT display instance

void setup()
{
#ifdef debug
  Serial.begin(baud_rate);
  Serial.println("Start");
#endif

  pinMode(StD_pin, INPUT);
  pinMode(Q1_pin, INPUT);
  pinMode(Q2_pin, INPUT);
  pinMode(Q3_pin, INPUT);
  pinMode(Q4_pin, INPUT);
  pinMode(relay_pin, OUTPUT);

  digitalWrite(relay_pin, LOW);

  // Display the splash screen
  tft.initR(INITR_18BLACKTAB);   // initialize a 1.8" TFT with ST7735S chip, black tab
  delay(tft_delay);
  tft.fillScreen(ST7735_BLUE);
  delay(tft_delay);
  tft.setRotation(1); // Set the screen rotation
  delay(tft_delay);
  tft.setTextWrap(false); // Turn off Text Wrap
  delay(tft_delay);
  tft.setTextSize(text_font_size); // Set the Font Size
  delay(tft_delay);
  tft.setTextColor(ST7735_WHITE); //Set the Text Color
  delay(tft_delay);
  tft.setCursor(40, 10);  //Set the Cursor and display the startup screen
  delay(tft_delay);
  tft.print("KW5GP");
  delay(tft_delay);
  tft.setTextSize(2);
  delay(tft_delay);
  tft.setCursor(45, 60);
  delay(tft_delay);
  tft.print("Arduino");
  delay(tft_delay);
  tft.setCursor(30, 80);
  delay(tft_delay);
  tft.print("TouchTone");
  delay(tft_delay);
  tft.setCursor(45, 100);
  delay(tft_delay);
  tft.print("Decoder");
  delay(tft_delay);

  delay(5000);
  clear_display();

}  // End Setup Loop

void loop()  // Main Program Loop
{
  // Read the StD pin

  Tone_Decoded = digitalRead(StD_pin);

  if (Tone_Decoded == HIGH)
  {
    if (previous_state == LOW)
    {
      decode_tone();
#ifdef debug
      Serial.print("   Tone_Decoded = ");
      Serial.print(digitalRead(Tone_Decoded));
      Serial.print("   Q1 = ");
      Serial.print(Q1);
      Serial.print("   Q2 = ");
      Serial.print(Q2);
      Serial.print("   Q3 = ");
      Serial.print(Q3);
      Serial.print("   Q4 = ");
      Serial.println(Q4);
#endif

      update_text();
      previous_state = HIGH;
    }
  }
  else
  {
    if (previous_state == HIGH)
    {
      previous_state = LOW;
    }
  }
}

void clear_display()  // Clears the TFT display
{
  tft.fillScreen(ST7735_BLUE);
  delay(tft_delay);
  tft.setTextSize(status_font_size); // Set the Font Size
  delay(tft_delay);
  tft.setCursor(40, 25);
  delay(tft_delay);
  tft.print("Decoded Digit");
  delay(tft_delay);
  update_relay_status();
  delay(tft_delay);
  tft.setTextSize(status_font_size);
  delay(tft_delay);
  tft.setCursor(40, 115);
  tft.print("Decoder Ready");
  delay(tft_delay);
  tft.setTextSize(text_font_size); // Set the Font Size
  delay(tft_delay);
  tft.setCursor(10, 40);
  delay(tft_delay);
}

void decode_tone()
{
  // Decodes the 4 bits of the Tone Decoder and returns the digit as a character

  // Read the decoded tone bits
  Q1 = digitalRead(Q1_pin);
  Q2 = digitalRead(Q2_pin);
  Q3 = digitalRead(Q3_pin);
  Q4 = digitalRead(Q4_pin);

  // Now convert it to a digit

  digit_value = (Q4 * 8) + (Q3 * 4) + (Q2 * 2) + Q1;

  /*    Decoded Values
        1-9 = Actual Value
        A = 13
        B = 14
        C = 15
        D = 0
        Asterisk (*) = 11
        # = 12
        0 = 10
  */

  switch (digit_value)
  {
    case (13):
      digit = 'A';
      break;

    case (14):
      digit = 'B';
      break;

    case (15):
      digit = 'C';
      break;

    case (0):
      digit = 'D';
      break;

    case (11):
      digit = '*';
      set_relay();

#ifdef debug
      Serial.println("Relay On");
#endif

      break;

    case (12):
      digit = '#';
      reset_relay();

#ifdef debug
      Serial.println("Relay On");
#endif

      break;

    case (10):
      digit = '0';
      break;

    default:
      digit = digit_value + '0';
      break;

  }

#ifdef debug
  Serial.print("Digit Value : ");
  Serial.print(digit_value);
  Serial.print("  Digit = ");
  Serial.println(digit);
  Serial.print("   TFT Char: ");
  Serial.print(digit);
#endif

}

void update_text() // Updates the tft text on the display
{
  if (tft_data.length() >= tft_line_length)  // Scroll one line of characters
  {
    tft_data = tft_data.substring(1);  // Remove the first character
    tft.setCursor(10, 40); // Move the cursor to the top line
    tft.fillRect(0, 40, 159, 30, ST7735_BLUE); // Clear the tft data area
    tft.print(tft_data); // Print the tft data string variable
  }

  tft_data = tft_data + digit;  // Add the character to the tft data string
  tft.print(digit); // display the received character

#ifdef debug
  if (tft_data.length() != 0)
  {
    Serial.print("digit: ");
    Serial.print(digit);
    Serial.print("   tft_data string length: ");
    Serial.print(tft_data.length());
    Serial.println(" ");
  }
#endif

}

void set_relay()
{
  digitalWrite(relay_pin, HIGH);
  relay_active = true;
  update_relay_status();
}

void reset_relay()
{
  digitalWrite(relay_pin, LOW);
  relay_active = false;
  update_relay_status();
}

void update_relay_status()
{
  tft.fillRect(0, 40, 159, 60, ST7735_BLUE); // Clear the tft data area
  delay(tft_delay);
  tft.setCursor(30, 80);
  delay(tft_delay);
  tft.setTextSize(relay_font_size);
  delay(tft_delay);
  tft.print("Relay ");
  delay(tft_delay);
  if (relay_active)
  {
    tft.print("On");
  }
  else
  {
    tft.print("Off");
  }
  delay(tft_delay);
  tft.setTextSize(text_font_size);
  delay(tft_delay);
  tft.setCursor(10, 40);
  delay(tft_delay);
  tft.print(tft_data);
  delay(tft_delay);
}
