/*
  AD9833 DDS Waveform Generator by Glen Popiel - KW5GP

  This program is free software: you can redistribute it and/or modify
  it under the terms of the version 3 GNU General Public License as
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

//#define debug // Uncomment this line to add debug output
#define baud_rate 115200

#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library
#include <MD_AD9833.h>  // AD9833 Library
#include <Encoder.h>  // Includer the Rotary Encoder Library

#define TFT_CS     10  // Assign the TFT CS to pin 10
#define TFT_RST    8  // Assign the TFT RST to pin 7
#define TFT_DC     9  // Assign the TFT DC to pin 6

#define text_font_size 3  // The TFT text display font size
#define freq_font_size 2  // The font size for the frequency display
#define status_font_size 1  // The font size for the status line
#define tft_delay 10  // set the TFT command delay to 10ms

#define FSYNC 5  // Define AD9833 FSYNC Pin as Digital Pin 2

#define Square_wave 6  // Define Square Wave Switch Input
#define Triangle_wave 7  // Define Triangle Wave Switch
#define min_freq 50  // Define the minimum frequency as 50Hz
#define max_freq 100000  // Define the maximum frequency as 1 MHz

const int encoder_PinA = 2;  // Pin A of the Rotary Encoder
const int encoder_PinB = 3;  // Pin B of the Rotary Encoder
const int encoder_Switch_Pin = 4;  // Encoder Pushbutton Switch input pin
boolean encoder_Switch;
const int debounceInterval = 500;  //disable step change for 500 ms between encoder pushbutton press

double freq = 1000; //32-bit global frequency variable
double step_size[] = {1, 10, 100, 1000, 10000}; // The step rate selected by the rotary encoder switch
double previous_frequency = freq;
int wave_type, previous_type = 0;
int encoder_Pos = 0;
int old_encoder_Pos = 0;
double current_step_size = step_size[2];
int step_index = 2;
unsigned long int delay_time = 0;
String wave_mode = "   Sine";
String spaces = "";

// Instantiate the AD9833
MD_AD9833  AD(FSYNC);

// Instantiate the TFT Display
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC, TFT_RST);  // Initialize the TFT display instance

// Instantiate the Rotary Encoder
Encoder Enc(encoder_PinA, encoder_PinB);

void setup()
{
#ifdef debug
  Serial.begin(baud_rate);
  Serial.println("Start");
#endif

  pinMode (encoder_Switch_Pin, INPUT_PULLUP);
  pinMode(Square_wave, INPUT_PULLUP);  // Set the Square_wave Control Pin as an Input
  pinMode(Triangle_wave, INPUT_PULLUP);  // Set the Triangle_wave Control Pin as an Input

  // Display the splash screen
  tft.initR(INITR_18BLACKTAB);   // initialize a 1.8" TFT with ST7735S chip, black tab
  delay(tft_delay);
  tft.fillScreen(ST7735_BLUE);
  delay(tft_delay);
  tft.setRotation(1); // Set the screen rotation
  delay(tft_delay);
  tft.setTextWrap(false); // Turn off Text Wrap
  delay(tft_delay);
  tft.setTextSize(text_font_size); // Set the Font Size
  delay(tft_delay);
  tft.setTextColor(ST7735_WHITE); //Set the Text Color
  delay(tft_delay);
  tft.setCursor(40, 10);  //Set the Cursor and display the startup screen
  delay(tft_delay);
  tft.print("KW5GP");
  delay(tft_delay);
  tft.setTextSize(2);
  delay(tft_delay);
  tft.setCursor(40, 60);
  delay(tft_delay);
  tft.print("Arduino");
  delay(tft_delay);
  tft.setCursor(35, 80);
  delay(tft_delay);
  tft.print("Waveform");
  delay(tft_delay);
  tft.setCursor(30, 100);
  delay(tft_delay);
  tft.print("Generator");
  delay(tft_delay);

  AD.begin(); // Start the DDS
  AD.setFrequency(MD_AD9833::CHAN_0, freq);

  delay(5000);
  clear_display();

}  // End Setup Loop

void loop()
{
  read_encoder(); // Read the encoder and set the Tx/Rx frequencies

  read_pushbutton();  // Read the Encoder Pushbutton switch and set Fast/Slow tuning mode

  read_wave_type(); // Read the Wave Type Switch and set the Wave Mode

}  // End Main Loop

void clear_display()  // Clears the TFT display
{
  tft.fillScreen(ST7735_BLUE);
  delay(tft_delay);
  tft.setTextSize(status_font_size);
  delay(tft_delay);
  tft.setCursor(60, 25);
  delay(tft_delay);
  tft.print("Frequency");
  delay(tft_delay);
  tft.setTextSize(freq_font_size);
  delay(tft_delay);
  tft.setCursor(5, 50);
  delay(tft_delay);
  // Add spaces to center the frequency display
  spaces = ("   ");
  if (freq < 100)
  {
    spaces = "    ";
  }
  if (freq >= 100 && freq <= 999)
  {
    spaces = "    ";
  }
  if (freq >= 1000 && freq <= 9999)
  {
    spaces = "   ";
  }
  if (freq >= 10000 && freq <= 99999)
  {
    spaces = "   ";
  }
  tft.print(spaces);
  delay(tft_delay);
  tft.print(freq, 0);
  delay(tft_delay);
  tft.print(" Hz");
  delay(tft_delay);
  tft.setTextSize(status_font_size);
  delay(tft_delay);
  tft.setCursor(40, 90);
  delay(tft_delay);
  tft.print(wave_mode);
  delay(tft_delay);
  tft.print(" Wave");
  delay(tft_delay);
  tft.setCursor(20, 115);
  spaces = " ";
  if (current_step_size == 1)
  {
    spaces = "  ";
  }
  if (current_step_size == 10)
  {
    spaces = "  ";
  }
  if (current_step_size == 100)
  {
    spaces = "  ";
  }
  if (current_step_size == 1000)
  {
    spaces = " ";
  }
  tft.print(spaces);
  delay(tft_delay);
  tft.print("Step Size: ");
  delay(tft_delay);
  tft.print(current_step_size, 0);
  tft.print(" Hz");
  tft.setTextSize(text_font_size); // Set the Font Size
  delay(tft_delay);
}

void read_encoder()
{
  // Read the Encoder
  encoder_Pos = Enc.read() / 4; // divide by 4 to match encoder detent
  if (encoder_Pos != old_encoder_Pos) // If the Encoder has changed update freq
  {
    if (encoder_Pos > old_encoder_Pos) // If we're increasing frequency
    {
      if (freq + current_step_size >= max_freq) // Limit to top end of band
      {
        freq = max_freq;
      } else
      {
        freq = freq + current_step_size;
      }

    } else
    {
      if (freq - current_step_size <= min_freq)// We're decreasing frequency, limit to minimum
      {
        freq = min_freq;
      } else
      {
        freq = freq - current_step_size;
      }
    }

    old_encoder_Pos = encoder_Pos;

    AD.setFrequency(MD_AD9833::CHAN_0, freq);

    tft.fillRect(0, 50, 159, 25, ST7735_BLUE); // Erase the lower portion of the TFT display
    delay(tft_delay);
    tft.setTextSize(freq_font_size);
    delay(tft_delay);
    tft.setCursor(5, 50);
    delay(tft_delay);
    // Add spaces to center the frequency display
    spaces = "  ";
    if (freq < 100)
    {
      spaces = "    ";
    }
    if (freq >= 100 && freq <= 999)
    {
      spaces = "    ";
    }
    if (freq >= 1000 && freq <= 9999)
    {
      spaces = "   ";
    }
    if (freq >= 10000 && freq <= 99999)
    {
      spaces = "   ";
    }
#ifdef debug
    Serial.print("Freq: ");
    Serial.println(freq);
#endif

    tft.print(spaces);
    delay(tft_delay);
    tft.print(freq, 0);
    delay(tft_delay);
    tft.print(" Hz");
    delay(tft_delay);
  }
}

void read_pushbutton()
{
  // Read the Encoder Pushbutton Switch
  encoder_Switch = digitalRead(encoder_Switch_Pin);
  if (encoder_Switch == LOW && millis() > delay_time) // Check to see if pressed
  {
    // if it's changed, toggle the step size but don't allow again for debounce time
    delay_time = millis() + debounceInterval; // if it's changed, toggle the step size but don't allow again for debounce time
    step_index = step_index + 1;
    if (step_index > 4) // We're at max step, reset to minimum step value
    {
      step_index = 0;
    }
    current_step_size = step_size[step_index];

#ifdef debug
    Serial.print("Step Size: ");
    Serial.println(current_step_size);
#endif

    tft.fillRect(0, 115, 159, 12, ST7735_BLUE); // Erase the lower portion of the TFT display
    delay(tft_delay);
    tft.setTextSize(status_font_size);
    delay(tft_delay);
    tft.setCursor(20, 115);
    delay(tft_delay);
    spaces = " ";
    if (current_step_size == 1)
    {
      spaces = "  ";
    }
    if (current_step_size == 10)
    {
      spaces = "  ";
    }
    if (current_step_size == 100)
    {
      spaces = "  ";
    }
    if (current_step_size == 1000)
    {
      spaces = " ";
    }
    tft.print(spaces);
    delay(tft_delay);
    tft.print("Step Size: ");
    delay(tft_delay);
    tft.print(current_step_size, 0);
    tft.print(" Hz");
  }
}

void read_wave_type()
{
  // Read the Wave selection switch and update the wave type if it has changed
  // Convert the Wave type switch input pins to a single binary value
  // Note, wave type cannot be 0 since the two I/O pins are attached to a SPDT switch
  wave_type = digitalRead(Square_wave) + (digitalRead(Triangle_wave) * 2);

  if (wave_type != previous_type)
  {
    // Switch has changed, update the wave type
    switch (wave_type)
    {
      case 1:
        // Square Wave
        AD.setMode(MD_AD9833::MODE_SQUARE1);
        wave_mode = " Square";

#ifdef debug
        Serial.println("Set Wave Type to Square Wave");
#endif

        break;

      case 2:
        // Triangle Wave
        AD.setMode(MD_AD9833::MODE_TRIANGLE);
        wave_mode = "Triangle";

#ifdef debug
        Serial.println("Set Wave Type to Triangle Wave");
#endif

        break;

      default:
        // Sine Wave
        AD.setMode(MD_AD9833::MODE_SINE);
        wave_mode = "  Sine";

#ifdef debug
        Serial.println("Set Wave Type to Sine Wave");
#endif

        break;
    }
    previous_type = wave_type;
    // Update the display

    tft.fillRect(0, 90, 159, 20, ST7735_BLUE); // Erase the lower portion of the TFT display
    delay(tft_delay);
    tft.setTextSize(status_font_size);
    delay(tft_delay);
    tft.setCursor(40, 90);
    delay(tft_delay);
    tft.print(wave_mode);
    delay(tft_delay);
    tft.print(" Wave");
    delay(tft_delay);
  }
}
