/*
  Arduino Mini Weather Station
  Author Glen Popiel, KW5GP
  ------------------------------------------
  Wind Sensor portion derived from Wind Sensor Example Sketch by Paul Badger and
  moderndevices.com
  ------------------------------------------
  Uses the Adafruit library for the BMP280 temperature & pressure sensor

  Uses the DFRobot library for the DHT20 humidity & temperature sensor

  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing products
  from Adafruit!

  Written by Limor Fried & Kevin Townsend for Adafruit Industries.
  BSD license, all text above must be included in any redistribution
  ------------------------------------------
*/

//#define debug // define debug to enable debug Serial Monitor output
#define baud_rate 115200

#include <Wire.h> // I2C Library
#include <Adafruit_BMP280.h>  //Adafruit BMP280 Library
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library
#include <DFRobot_DHT20.h>

#define analogPinForRV    1   // The Wind Sensor RV Pin
#define analogPinForTMP   0   // The Wind Sensor TMP pin

#define TFT_CS     10  // Assign the TFT CS to pin 10
#define TFT_RST    8  // Assign the TFT RST to pin 7
#define TFT_DC     9  // Assign the TFT DC to pin 6

#define tft_delay 10  // set the TFT command delay to 10ms
#define update_delay 10000

const float zeroWindAdjustment =  .2; // negative numbers yield smaller wind speeds and vice versa
int TMP_Therm_ADunits;  //temp termistor value from wind sensor
float RV_Wind_ADunits;    //RV output from wind sensor
float RV_Wind_Volts;
float TempCtimes100;
float TempF;
float zeroWind_ADunits;
float zeroWind_volts;
float WindSpeed_MPH;
unsigned bmp_status;
float bmp_temp;
float bmp_pressure;
float bmp_inches;
float dht_temp;
float dht_humidity;

// Instantiate the TFT Display
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC, TFT_RST);  // Initialize the TFT display instance

// Instantiate the BMP280 sensor
Adafruit_BMP280 bmp; // use BMP280 I2C interface

// Instantiate the DHT20 sensor
DFRobot_DHT20 dht20;

void setup()
{
#ifdef debug
  Serial.begin(baud_rate);
  Serial.println("Start");
#endif

  dht20.begin(); // Start the DHT20 sensor

  // Display the splash screen
  tft.initR(INITR_18BLACKTAB);   // initialize a 1.8" TFT with ST7735S chip, black tab
  delay(tft_delay);
  clear_display(); // Clear the display - fill with BLACK background
  delay(tft_delay);
  tft.setRotation(1); // Set the screen rotation
  delay(tft_delay);
  tft.setTextWrap(false); // Turn off Text Wrap
  delay(tft_delay);
  tft.setTextSize(3); // Set the Font Size
  delay(tft_delay);
  tft.setTextColor(ST7735_WHITE); //Set the Text Color
  delay(tft_delay);
  tft.setCursor(40, 10);  //Set the Cursor and display the startup screen
  delay(tft_delay);
  tft.print("KW5GP");
  delay(tft_delay);
  tft.setTextSize(2);
  delay(tft_delay);
  tft.setCursor(40, 60);
  delay(tft_delay);
  tft.print("Arduino");
  delay(tft_delay);
  tft.setCursor(40, 80);
  delay(tft_delay);
  tft.print("Weather");
  delay(tft_delay);
  tft.setCursor(40, 100);
  delay(tft_delay);
  tft.print("Station");
  delay(tft_delay);

  // Start the BMP280 - You may need to use the alternate address version with
  // non-Adafruit BMP Modules as I had to here with a generic BMP280 module

  //  status = bmp.begin();
  bmp_status = bmp.begin(BMP280_ADDRESS_ALT, BMP280_CHIPID);

#ifdef debug

  if (!bmp_status)
  {
    Serial.println(F("Could not find a valid BMP280 sensor, check wiring or "
                     "try a different address!"));
    Serial.print("SensorID was: 0x"); Serial.println(bmp.sensorID(), 16);
    Serial.print("        ID of 0xFF probably means a bad address, a BMP 180 or BMP 085\n");
    Serial.print("   ID of 0x56-0x58 represents a BMP 280,\n");
    Serial.print("        ID of 0x60 represents a BME 280.\n");
    Serial.print("        ID of 0x61 represents a BME 680.\n");
    while (1) delay(10);
  }

  Serial.print("BMP280 ALT Address: ");
  Serial.print(BMP280_ADDRESS_ALT);

  Serial.print("   BMP280 CHIP ID: ");
  Serial.println(BMP280_CHIPID);
#endif

  /* Default settings from datasheet. */
  bmp.setSampling(Adafruit_BMP280::MODE_NORMAL,     /* Operating Mode. */
                  Adafruit_BMP280::SAMPLING_X2,     /* Temp. oversampling */
                  Adafruit_BMP280::SAMPLING_X16,    /* Pressure oversampling */
                  Adafruit_BMP280::FILTER_X16,      /* Filtering. */
                  Adafruit_BMP280::STANDBY_MS_500); /* Standby time. */

  delay(5000);
}  // End Setup Loop

void loop()  // Main Program Loop
{
  get_bmp_data(); // Read the BMP280 and store the data into variables
  get_wind_data();  // Read the Wind Speed sensor and store the data into variables
  get_dht_data(); // Read the DHT20 sensor and store the data into variables
  update_display(); // Update the display with the new data

  delay(update_delay);
}

void update_display()  // Updates the mode on the display
{
#ifdef debug
  Serial.println("Wind Speed Sensor Data");
  Serial.print("TMP volts ");
  Serial.print(TMP_Therm_ADunits * 0.0048828125);

  Serial.print(" RV volts ");
  Serial.print((float)RV_Wind_Volts);

  Serial.print("\t  TempC*100 ");
  Serial.print(TempCtimes100 );

  Serial.print("\t  TempF ");
  Serial.print(TempF, 2);

  Serial.print("   ZeroWind volts ");
  Serial.print(zeroWind_volts);

  Serial.print("   WindSpeed MPH ");
  Serial.println((float)WindSpeed_MPH);

  Serial.println("BMP280 Sensor Data");

  Serial.print("BMP Temp: ");
  Serial.print(bmp_temp, 2);
  Serial.print("   Pressure: ");
  Serial.print(bmp_pressure, 4);
  Serial.print("  Inches: ");
  Serial.println(bmp_inches, 4);

  Serial.println("DHT22 Sensor Data");

  Serial.print("DHT Temp: ");
  Serial.print(dht_temp, 2);
  Serial.print("   Humidity: ");
  Serial.println(dht_humidity, 4);

#endif

  // Display the data
  clear_display();
  delay(tft_delay);

  tft.setCursor(35, 5);
  delay(tft_delay);
  tft.print("Weather Station");
  delay(tft_delay);
  tft.setCursor(10, 55);
  delay(tft_delay);
  tft.print("DHT Temp: ");
  delay(tft_delay);
  tft.print(dht_temp, 2);
  delay(tft_delay);
  tft.print(" F");
  delay(tft_delay);
  tft.setCursor(10, 65);
  delay(tft_delay);
  tft.print("Humidity: ");
  delay(tft_delay);
  tft.print(dht_humidity, 1);
  delay(tft_delay);
  tft.print(" %");
  tft.setCursor(10, 75);
  delay(tft_delay);
  tft.print("BMP Temp: ");
  delay(tft_delay);
  tft.print(bmp_temp, 2);
  delay(tft_delay);
  tft.print(" F");
  delay(tft_delay);
  tft.setCursor(10, 85);
  delay(tft_delay);
  tft.print("Pressure: ");
  delay(tft_delay);
  tft.print(bmp_inches, 4);
  delay(tft_delay);
  tft.setCursor(10, 95);
  delay(tft_delay);
  tft.print("Wind Temp: ");
  delay(tft_delay);
  tft.print(TempF, 2);
  delay(tft_delay);
  tft.print(" F");
  delay(tft_delay);
  tft.setCursor(10, 105);
  delay(tft_delay);
  tft.print("Wind Speed: ");
  delay(tft_delay);
  tft.print(WindSpeed_MPH);
  delay(tft_delay);
}


void clear_display()  // Clears the TFT display
{
  tft.fillScreen(ST7735_BLUE);
  delay(tft_delay);
  tft.setTextSize(1); // Set the Font Size
  delay(tft_delay);
}

void get_wind_data()  // Reads the data from the Wind Speed sensor
{
  // This portion is duplicated from the example sketch
  TMP_Therm_ADunits = analogRead(analogPinForTMP);
  RV_Wind_ADunits = analogRead(analogPinForRV);
  RV_Wind_Volts = (RV_Wind_ADunits *  0.0048828125);
  TempCtimes100 = (0.005 * ((float)TMP_Therm_ADunits * (float)TMP_Therm_ADunits)) - (16.862 * (float)TMP_Therm_ADunits) + 9075.4;
  TempF = ((TempCtimes100 * 9 / 5) + 3200) / 100;

  zeroWind_ADunits = -0.0006 * ((float)TMP_Therm_ADunits * (float)TMP_Therm_ADunits) + 1.0727 * (float)TMP_Therm_ADunits + 47.172; //  13.0C  553  482.39

  zeroWind_volts = (zeroWind_ADunits * 0.0048828125) - zeroWindAdjustment;

  // This from a regression from data in the form of
  // Vraw = V0 + b * WindSpeed ^ c
  // V0 is zero wind at a particular temperature
  // The constants b and c were determined by some Excel wrangling with the solver.

  //We can't have a negative number in the following calculation otherwise we get a "nan" on the display
  // You can also adjust the zeroWindAdjustment constant to fine tune the zeroWind_volts
  
  if (RV_Wind_Volts - zeroWind_volts < 0)
  {
    zeroWind_volts = RV_Wind_Volts;
  }
  else
  {
    WindSpeed_MPH =  pow(((RV_Wind_Volts - zeroWind_volts) / .2300) , 2.7265);
  }
}

void get_bmp_data() // Reads the BMP280 sensor
{
  bmp_temp = bmp.readTemperature();
  // convert temp to Farenheit
  bmp_temp = (bmp_temp * 9 / 5) + 32;

  bmp_pressure = bmp.readPressure() / 100;
  bmp_inches = bmp_pressure * .02953; // convert hPa to inches of Mercury
}

void get_dht_data() // Reads the DHT20 sensor
{
  dht_temp = dht20.getTemperature();
  // convert temp to Farenheit
  dht_temp = (dht_temp * 9 / 5) + 32;
  dht_humidity = dht20.getHumidity() * 100;
}
